import sys
from queue import Empty
from multiprocessing import Process
from copy import deepcopy

from cml.shared.logger import ConstructionInfo


class BackgroundConstructor(Process):
    def __init__(self, models, logger, settings, input_queue, output_queue):
        super(BackgroundConstructor, self).__init__(name="BackgroundConstructor")
        self.models = models
        self.logger = logger
        self.settings = settings
        self.input_queue = input_queue
        self.output_queue = output_queue

    def run(self):
        while 1:
            try:
                learnblock = self.input_queue.get(timeout=30)
                for args in self.prepare_args(learnblock):
                    logger_info = ConstructionInfo()
                    learnblock, model, min_category_size = args
                    if learnblock.labeled():
                        logger_info.state = ConstructionInfo.State.PASSED
                        self.output_queue.put((logger_info, learnblock))
                    else:
                        trained_model = model.train(learnblock)
                        for cluster, size in trained_model.cluster_sizes.items():
                            if size < min_category_size:
                                break
                        else:
                            labels = trained_model.get_labels()
                            learnblock.set_column("Z", labels)
                            labeled_learnblock = learnblock
                            labeled_learnblock.n_cluster = model.cluster
                            purpose = "{}{:02}".format(model.abbreviation,
                                                       model.cluster)
                            labeled_learnblock.subject = purpose

                            # Fill the logger object
                            logger_info.n_cluster = trained_model.cluster
                            logger_info.subject = trained_model.subject
                            logger_info.state = ConstructionInfo.State.CONSTRUCTED

                            self.output_queue.put((logger_info, labeled_learnblock))

                        if logger_info.state is None:
                            logger_info.state = ConstructionInfo.State.DISCARDED
                            self.output_queue.put((logger_info, None))

            except Empty:
                sys.exit()

    def prepare_args(self, learnblock):
        for ml_model in self.models:
            for complexity in range(2, self.settings.max_categories+1):
                ml_model.cluster = complexity
                yield deepcopy(learnblock), deepcopy(ml_model), complexity
