from cml.domain.reconstruction import (
    split,
    valid_reconstructed,
    calc_reliability,
    Metadata,
    PragmaticMachineLearningModel)


def reconstruct_in_parallel(*args):
    dto = args[0]
    train_block, eval_block = split(dto.learnblock, dto.settings.reliability_sample)
    trained_model = dto.model.train(train_block.as_numpy_array(),
                                    [i for i in train_block.get_column("Z")])
    if valid_reconstructed(trained_model,
                           "conceptual",
                           dto.settings.min_test_accuracy,
                           dto.settings.max_test_error_avg,
                           dto.settings.max_test_error_max):
        reliability = calc_reliability(trained_model, dto.learnblock, "nominal")
        if reliability >= dto.settings.min_reliability:
            trained_model.reliability = reliability
            uid = ".".join([dto.abv,
                            str(dto.tier),
                            dto.learnblock.subject])
            meta = Metadata(dto.abv,
                            dto.tier,
                            0,
                            dto.learnblock.indices(),
                            dto.learnblock.columns(),
                            dto.learnblock.get_column("Z"),
                            dto.learnblock.min_timestamp(),
                            dto.learnblock.max_timestamp(),
                            (dto.model.subject,),
                            (uid,))
            return reliability, PragmaticMachineLearningModel(
                meta, trained_model, dto.learnblock)

    return None, None
